{ config, lib, pkgs, ... }:

let 
  cfg = config.jade.desktop;
  window-screenshot = pkgs.writeShellScriptBin "window-screenshot.sh" (builtins.readFile ../scripts/desktop/window-screenshot.sh);
  # desktop-ctl = pkgs.writeShellScriptBin "desktop-ctl.sh" (builtins.readFile ../scripts/desktop/desktop-ctl.sh);
  desktop-ctl = import ../scripts/desktop/desktopctl.nix { pkgs = pkgs; };
  permaclip = import ../scripts/desktop/permaclip { pkgs = pkgs; };
  em-record = pkgs.writeShellScriptBin "em-record.sh" (builtins.readFile ../scripts/desktop/macros/record.sh);
  em-play = pkgs.writeShellScriptBin "em-play.sh" (builtins.readFile ../scripts/desktop/macros/play.sh);
  em-play-loop = pkgs.writeShellScriptBin "em-play-loop.sh" (builtins.readFile ../scripts/desktop/macros/play-loop.sh);
  ocr-screenshot = pkgs.writeShellScriptBin "ocr-screenshot.sh" (builtins.readFile ../scripts/desktop/ocr-screenshot.sh);
  # rofi calculator copied from https://github.com/barbuk/menu-qalc
  menu-qalc = pkgs.writeShellScriptBin "menu-qalc" (builtins.readFile ../scripts/desktop/menu-qalc.sh);
in with lib; {
  options.jade.desktop = {
    enable = mkEnableOption "Enable the i3 setup";
    compositing = mkEnableOption "Enable compositing via picom";
  };
  config = mkIf cfg.enable {
    home-manager.users.jade = { pkgs, ... }: {
      home.packages = with pkgs; [
        # external
        libnotify
        i3lock-fancy rofi rofimoji volumeicon feh xorg.xinput dunst
        arandr lxappearance gruvbox-dark-gtk gruvbox-dark-icons-gtk 
        gruvterial-theme flameshot tesseract5 imagemagick xclip polybar kitty
        xkeysnail gtk-engine-murrine playerctl xmacro pa_applet brightnessctl 
        libqalculate # needed by menu-qalc
        # custom scripts
        window-screenshot desktop-ctl em-record em-play em-play-loop ocr-screenshot menu-qalc
        permaclip

        # themes
        gruvbox-dark-gtk gruvbox-dark-icons-gtk
      ];
      # i3 {{{
      xsession = {
        enable = true;
        windowManager.i3 = {
          enable = true;
          config = {
            bars = [];
            terminal = "kitty";
            workspaceAutoBackAndForth = true;
            focus = {
              followMouse = false;
            };
            # Startup {{{
              startup = [
                # Keyboard remapping
                { command = "xhost +"; }
                { command = "xkeysnail ${../other/xkeysnail.py}"; always = true; }
                { command = "sleep 4 && setxkbmap -layout us -variant altgr-intl"; always = true; }
                { command = "feh --bg-scale ${../other/wallpaper.jpg}"; always = true; notification = false; }
                { command = "picom --experimental-backend"; }
                { command = "pkill nm-applet; nm-applet"; always = true; }
                { command = "pkill pa-applet; pa-applet"; always = true; } 
              ];
            # }}}
            # Assigns {{{
            assigns = {
              "1" = [ { class = "discord"; } { class = "nheko"; } ];
              "2" = [ { class = "librewolf"; } ];
            };
            # }}}
            # Keyboard shortcuts {{{
            modifier = "Mod4";
            modes = {
              resize = with {
                # mod = builtins.break config.xsession.windowManager.i3.config.modifier;
                mod = builtins.break "Mod4";
              }; {
                aaa = builtins.break "a";
                "${mod}+h" = "resize shrink width 10 px or 10 ppt";
                "${mod}+j" = "resize grow height 10 px or 10 ppt";
                "${mod}+k" = "resize shrink height 10 px or 10 ppt";
                "${mod}+l" = "resize grow width 10 px or 10 ppt";

                "Return" = "mode default";
                "Escape" = "mode default";
                "${mod}+r" = "mode default";
              };
            };
            keybindings = with {
                 #mod = config.xsession.windowManager.i3.config.modifier;
                 mod = "Mod4";
            }; lib.mkOptionDefault {
              # switch window focus
              "${mod}+h" = "focus left";
              "${mod}+j" = "focus down";
              "${mod}+k" = "focus up";
              "${mod}+l" = "focus right";
              # move windows
              "${mod}+Shift+h" = "move left";
              "${mod}+Shift+j" = "move down";
              "${mod}+Shift+k" = "move up";
              "${mod}+Shift+l" = "move right";
              # layout shit 
              "${mod}+shift+semicolon" = "split h";
              "${mod}+semicolon" =  "split v";
              "${mod}+f" = "fullscreen toggle";
              "${mod}+Shift+w" = "layout tabbed";
              "${mod}+e" = "layout toggle split";
              "${mod}+Shift+space" = "floating toggle";
              # focus parents/children
              "${mod}+Shift+a" = "focus parent";
              "${mod}+Shift+c" = "focus child";
              # screenshot
              "${mod}+w" = "exec window-screenshot.sh";
              "${mod}+s" = "exec flameshot gui -c -p $HOME/Pictures/screenshots";
              "${mod}+a" = "exec flameshot screen -c -p $HOME/Pictures/screenshots";
              "${mod}+t" = "exec ocr-screenshot.sh";
              # rofi fuckery
              "${mod}+d" = "exec --no-startup-id rofi -show drun -theme ${../other/rofi-themes/applauncher.rasi}";
              "${mod}+space" = "exec --no-startup-id -show window";
              "${mod}+i" = "exec --no-startup-id rofimoji -f alchemical_symbols anatolian_hieroglyphs emojis braille_patterns box_drawing chess_symbols emoticons geometric_shapes gothic greek_extended math mathematical_alphanumeric_symbols mathematical_operators miscellaneous_symbols miscellaneous_mathematical_symbols-a miscellaneous_mathematical_symbols-b miscellaneous_symbols_and_arrows miscellaneous_symbols_and_pictographs miscellaneous_technical modi modifier_tone_letters musical_symbols nerd_font number_forms shorthand_format_controls specials variation_selectors vertical_forms -a copy";
              "${mod}+Shift+e" = "exec --no-startup-id rofi -show \"desktopctl\" -modes \"desktopctl:${desktop-ctl.outPath}/bin/desktopctl\" -theme ${../other/rofi-themes/applauncher.rasi}";
              "${mod}+m" = "exec --no-startup-id menu-qalc";

              # audio
              "XF86AudioRaiseVolume" = "exec --no-startup-id pactl set-sink-volume @DEFAULT_SINK@ +10% && $refresh_i3status";
              "XF86AudioLowerVolume" = "exec --no-startup-id pactl set-sink-volume @DEFAULT_SINK@ -10% && $refresh_i3status";
              "XF86AudioMute" = "exec --no-startup-id pactl set-sink-mute @DEFAULT_SINK@ toggle && $refresh_i3status";
              "XF86AudioMicMute" = "exec --no-startup-id pactl set-source-mute @DEFAULT_SOURCE@ toggle && $refresh_i3status";

              "XF86AudioNext" = "exec playerctl next";
              "XF86AudioPrev" = "exec playerctl previous";
              "XF86AudioPause" = "exec playerctl pause";
              "XF86AudioPlay" = "exec playerctl play";
              "XF86AudioStop" = "exec playerctl stop";

              "XF86MonBrightnessUp" = "exec brightnessctl set 5%+";
              "XF86MonBrightnessDown" = "exec brightnessctl set 5%-";

              # macros
              "${mod}+q" = "exec em-record.sh";
              "${mod}+p" = "exec em-play.sh";
              "${mod}+Shift+p" = "exec em-play-loop.sh";

              # permaclip
              "${mod}+c" = "rofi -show permaclip -modes \"permaclip:${permaclip.outPath}/bin/pc-set\" -theme gruvbox-dark";
              "${mod}+v" = "rofi -show permaclip -modes \"permaclip:${permaclip.outPath}/bin/pc-get\" -theme gruvbox-dark";
            };
            # }}}
            # Visuals {{{
            fonts = {
              names = [ "Montserrat" ];
              style = "Regular";
              size = 9.0;
            };
            gaps = {
              top = 24;
              inner = 15;
              outer = 0;
              smartGaps = false;
              smartBorders = "no_gaps";
            };
            colors = {
              background = "#1d2021";
              focused = { background = "#282828"; border = "#504945"; childBorder = "#7c6f64"; indicator = "#504945"; text = "#ebdbb2"; };
              focusedInactive = { background = "#1d2021"; border = "#504945"; childBorder = "#665c54"; indicator = "#664c54"; text = "#d5c4a1"; };
              placeholder = { background = "#1d2021"; border = "#00ff00"; childBorder = "#504945"; indicator = "#504945"; text = "#928374"; };
              unfocused = { background = "#1d2021"; border = "#3c3836"; childBorder = "#504945"; indicator = "#504945"; text = "#bdae93"; };
              urgent = { background = "#9d0006"; border = "#cc241d"; childBorder = "#3c3836"; indicator = "#fb4943"; text = "#ebdbb2"; };
            };
            window = {
              border = 2;
              titlebar = false;
            };
            # }}}
          };
        };
      };
      # }}}
      # Compositing {{{
      services.picom = mkIf config.jade.desktop.compositing {
        enable = true;
        experimentalBackends = true;
        backend = "glx";

        shadow = true;
        shadowOffsets = [ 25 15 ];
        shadowOpacity = 0.045;
        shadowExclude = [
          "class_g = 'Conky'"
          "class_g ?= 'Notify-osd'"
          "class_g = 'Cairo-clock'"
          "_GTK_FRAME_EXTENTS@:c"
          "class_g != 'Rofi'"
        ];

        vSync = true;
        opacityRules = [
          "90:class_g = 'kitty'"
          "80:class_g = 'Signal'"
          "80:class_g = 'Rofi'"
          "80:class_g = 'discord'"
          "80:class_g = 'Mailspring'"
          "85:class_g = 'nheko'"
          "75:class_g = 'obsidian'"
        ];
        settings = {
          # blur
          #"detect-client-opacity = true;"
          #"blur-method = \"gaussian\""
          #"blur-size = 30"
          #"blur-deviation = 15"
          #"blur-background-fixed = true"
          "detect-client-opacity" = true;
          "blur-method" = "gaussian";
          "blur-size" = 30;
          "blur-deviation" = 15;
          "blur-background-fixed" = true;

          # fading rofi
          "fading" = true;
          "fade-in-step" = 0.25;
          "fade-out-step" = 0.2;
          "fade-delta" = 20;
          "fade-exclude" = [ "class_g != 'Rofi'" ];
          # use damage information
          "use-damage" = true; 
        };
      };
      # }}}
      # Polybar {{{
        services.polybar = {
          enable = true;

          script = "pkill polybar; polybar &";
          settings = {
            colors = {
              background = "#282828";
              background-alt = "#3c3836";
              foreground = "#ebdbb2";
              primary = "#d65d0e";
              secondary = "#fe8019";
              alert = "#cc241d";
              disabled = "#504945";
            };
            "bar/status" = {
              # Style
              width = "100%";
              height = "24px";
              radius = 0;
              tray-position = "center";
              background = "\${colors.background}";
              foreground = "\${colors.foreground}";
              line-size = "4px";
              border-color = "#00000000";
              padding-left = 0;
              padding-right = 1;
              module-margin = 1;
              separator = "|";
              separator-foreground = "\${colors.disabled}";
              font-0 = "FiraCode Nerd Font";
              modules-left = "xworkspaces xwindow";
              modules-right = "memory cpu wlan battery date";
              cursor-click = "pointer";
              cursor-scroll = "ns-resize";
              enable-ipc = true;
              wm-restack = "i3";
              override-redirect = true;
            };

            "module/xworkspaces" = {
              type = "internal/xworkspaces";
              label-active = "";
              label-active-padding = 1;
              label-occupied = "";
              label-occupied-padding = 1;
              label-urgent = "";
              label-urgent-foreground = "\${colors.alert}";
              label-urgent-padding = 1;
              label-empty = "";
              label-empty-foreground = "\${colors.disabled}";
              label-empty-padding = 1;
            };

            "module/xwindow" = {
              type = "internal/xwindow";
              label = "%title:0:60:...%";
            };

            "module/memory" = {
              type = "internal/memory";
              interval = 2;
              format-prefix = "﬙ ";
              format-prefix-foreground = "\${colors.primary}";
              label = "%percentage_used:2%%";
            };

            "module/cpu" = {
              type = "internal/cpu";
              interval = "2";
              format-prefix = " ";
              format-prefix-foreground = "\${colors.primary}";
              label = "%percentage:2%%";
            };

            "network-base" = {
              type = "internal/network";
              interval = 5;
              format-connected = "<label-connected>";
              format-disconnected = "<label-disconnected>";
              label-disconnected = "%{F#F0C674}%ifname%%{F#707880} disconnected";
            };

            "module/wlan" = {
              "inherit" = "network-base";
              interface-type = "wireless";
              label-connected = "%{F#F0C674}%ifname%%{F-} %essid% %local_ip%";
            };

            "module/battery" = {
              type = "internal/battery";
              poll-interval = 5;
              full-at = 96;
              battery = "BAT0";
              adapter = "ADP1";
              time-format = "%H:%M";
              label-charging = "%percentage%% %time%";
              format-charging = "<ramp-capacity> ﮣ <label-charging>";
              label-discharging = "%percentage%% %time%";
              format-discharging = "<ramp-capacity> <label-discharging>";
              label-full = "%percentage%%";
              format-full = " <label-full>";
              label-low = "%percentage%%";
              format-low = " <label-low>";
              ramp.capacity = [ "" "" "" "" "" "" "" "" "" "" ];
            };

            "module/date" = {
              type = "internal/date";
              interval = 1;
              date = "%Y-%m-%d %H:%M:%S";
              label = "%date%";
              label-foreground = "\${colors.foreground}";
            };

            "settings" = {
              screenchange-reload = true;
              pseudo-transparency = true;
            };
          };
        };
      # }}}
      # gtk {{{
      gtk = {
        enable = true;
        cursorTheme = {
          package = pkgs.phinger-cursors;
          name = "phinger-cursors";
          size = 30;
        };
        font = { 
          package = pkgs.montserrat;
          name = "Montserrat";
          size = 11;
        };
        iconTheme = {
          package = pkgs.gruvbox-dark-icons-gtk;
          name = "gruvbox-dark-icons";
        };
        theme = {
          package = pkgs.gruvbox-dark-gtk;
          name = "gruvbox-dark";
        };
      };
      # }}}
      # Qt {{{
      qt = {
        enable = true;
        platformTheme = "gtk";
      };
      # }}}
    };
  };
}
